"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const common_1 = require("@fabrix/fabrix/dist/common");
const lodash_1 = require("lodash");
const validator_1 = require("./validator");
const elasticsearch = require("elasticsearch");
const config = require("./config/index");
const pkg = require("../package.json");
const api = require("./api/index");
class ElasticsearchSpool extends common_1.Spool {
    constructor(app) {
        super(app, {
            config: config,
            api: api,
            pkg: pkg
        });
        this.extensions = {
            elasticClient: {
                get: () => {
                    return this.client;
                },
                set: (newElasticSearchClient) => {
                    throw new Error('elasticClient can not be set through FabrixApp, check spool-elasticsearch instead');
                },
                enumerable: true,
                configurable: true
            }
        };
    }
    validate() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!lodash_1.isObject(this.app.config.get('elasticsearch'))) {
                return Promise.reject(new Error('No configuration found at config.elasticsearch!'));
            }
            if (!lodash_1.isObject(this.app.config.get('elasticsearch.connection'))) {
                return Promise.reject(new Error('No connection configuration defined!'));
            }
            const stores = this.app.config.get('stores');
            if (stores && Object.keys(stores).length === 0) {
                this.app.log.warn('No store configured at config.stores, models will be ignored');
            }
            const models = this.app.config.get('models');
            if (models && Object.keys(models).length === 0) {
                this.app.log.warn('No models configured at config.models, models will be ignored');
            }
            return Promise.all([
                validator_1.Validator.validateStoresConfig(stores),
                validator_1.Validator.validateModelsConfig(models),
                validator_1.Validator.validateElasticConfig(this.app.config.get('elasticsearch'))
            ]);
        });
    }
    configure() {
        if (!this.app.config.get('elasticsearch.connection.log')) {
            this.app.config.set('elasticsearch.connection.log', 'warning');
        }
    }
    initialize() {
        return __awaiter(this, void 0, void 0, function* () {
            this.client = new elasticsearch.Client(lodash_1.clone(this.app.config.get('elasticsearch.connection')));
            if (!this.app.config.get('elasticsearch.validateConnection')) {
                return this.migrate();
            }
            return new Promise((resolve, reject) => {
                this.client.ping((err) => {
                    if (err) {
                        return reject(err);
                    }
                    return this.migrate()
                        .then(() => {
                        resolve();
                    })
                        .catch(_err => {
                        reject(_err);
                    });
                });
            });
        });
    }
    unload() {
        if (!this.app.elasticClient || !lodash_1.isFunction(this.app.elasticClient.close)) {
            return;
        }
        return this.app.elasticClient.close();
    }
    migrate() {
        return __awaiter(this, void 0, void 0, function* () {
            const SchemaMigrationService = this.app.services.ElasticsearchSchemaMigrationService;
            return SchemaMigrationService.migrateDB([this.app.elasticClient]);
        });
    }
}
exports.ElasticsearchSpool = ElasticsearchSpool;
